@@ -9,19 +9,26 @@ my @stubs = qw/
 cubic2erect
 enblend-mask
 enblend-svg
+enfuse-mask
 erect2cubic
 erect2mercator
 erect2planet
 erect2qtvr
+gigastart
 jpeg2qtvr
 match-n-shift
 nona-mask
 pafextract
 panostart
 process-masks
+pto2mk2
+ptoanchor
+ptobind
 ptocentre
+ptochain
 ptoclean
 ptodummy
+ptofill
 ptoget
 ptograph
 ptoinfo
