@@ -22,12 +22,16 @@ One line per point
 Created automatically by optimizer
 
   i0           image
+  c0           control point
   x363.726     starting x point position
   y125.738     starting y point position
   X363.838     morphed x point position
   Y125.618     morphed y point position
+  D12.34       control point error
+  Dx7.89       control point error in horisontal direction
+  Dy-9.49      control point error in vertical direction
 
-  C i0  x363.726 y125.738 X363.838 Y125.618
+  C i0 c0  x363.726 y125.738 X363.838 Y125.618  D12.34 Dx7.89 Dy-9.49
 
 =cut
 
@@ -36,7 +40,7 @@ sub _defaults
     my $self = shift;
 }
 
-sub _valid { return '^([ixXyY])(.*)' }
+sub _valid { return '^([icxXyY]|D[xy]?)(.*)' }
 
 sub Identifier
 {
